from django.test import TestCase, RequestFactory,override_settings
from django.shortcuts import reverse
from .views import IndexView,CategoryDetail
from . import models
from django.contrib.auth.models import User
# Create your tests here.

class IndexViewTestCase(TestCase):

    @override_settings(FAQ_SETTINGS=["no_category"])
    def test_get_template_names_no_categories(self):
        "gets correct template when not using categories"
        request = RequestFactory().get(reverse("faq:index_view"))
        view = IndexView()
        view.setup(request)

        self.assertEquals(view.get_template_names(),"faq/questions_list.html")
        self.assertNotEquals(view.get_template_names(),"faq/categories_list.html")

    @override_settings(FAQ_SETTINGS=[])
    def test_get_template_names_categories(self):
        "gets correct template when not using categories"
        request = RequestFactory().get(reverse("faq:index_view"))
        view = IndexView()
        view.setup(request)

        self.assertNotEquals(view.get_template_names(), "faq/questions_list.html")
        self.assertEquals(view.get_template_names(), "faq/categories_list.html")

    @override_settings(FAQ_SETTINGS=["no_category"])
    def test_get_queryset_no_categories(self):
        "gets correct query set when not using categories"
        request = RequestFactory().get(reverse("faq:index_view"))
        view = IndexView()
        view.setup(request)

        models.Question.objects.create(question="?")
        models.Question.objects.create(question="the?")
        view.get_queryset()


        self.assertEqual(view.get_queryset().first(), models.Question.objects.first())
        self.assertNotEqual(view.get_queryset().first(), models.Question.objects.last())
        self.assertNotEqual(view.get_queryset().first(), models.Category.objects.first())

    @override_settings(FAQ_SETTINGS=[])
    def test_get_queryset_categories(self):
        "gets correct query set when using categories"
        request = RequestFactory().get(reverse("faq:index_view"))
        view = IndexView()
        view.setup(request)

        category= models.Category.objects.create(name="category", description="this is a category")
        models.Category.objects.create(name="category 2", description="this is a category")
        models.Question.objects.create(question="category question", category=category)
        models.Question.objects.create(question="category question 2", category=category)
        models.Question.objects.create(question="question not in category")


        self.assertEqual(view.get_queryset().first(), models.Category.objects.first())
        self.assertNotEqual(view.get_queryset().first(), models.Question.objects.first())
        self.assertNotEqual(view.get_queryset().first(), models.Category.objects.last())

    @override_settings(FAQ_SETTINGS=[])
    def test_get_context_object_name_categories(self):
        "gets correct template variable when using categories"
        request = RequestFactory().get(reverse("faq:index_view"))
        view = IndexView()
        view.setup(request)

        self.assertEqual(view.get_context_object_name([]), "categories")
        self.assertNotEqual(view.get_context_object_name([]), "questions")

    @override_settings(FAQ_SETTINGS=["no_category"])
    def test_get_context_object_name_no_categories(self):
        "gets correct template variable when not using categories"
        request = RequestFactory().get(reverse("faq:index_view"))
        view = IndexView()
        view.setup(request)

        self.assertEqual(view.get_context_object_name([]), "questions")
        self.assertNotEqual(view.get_context_object_name([]), "categories")

    @override_settings(FAQ_SETTINGS=["no_category"])
    def test_get_context_data_not_using_categories(self):
        "gets context data correctly when not using categories"
        request = RequestFactory().get(reverse("faq:index_view"))
        view = IndexView()
        view.object_list = view.get_queryset()
        view.setup(request)

        self.assertIn("can_add_question",view.get_context_data())

    @override_settings(FAQ_SETTINGS=[])
    def test_get_context_data_using_categories(self):
        "gets context data correctly when using categories"
        request = RequestFactory().get(reverse("faq:index_view"))
        view = IndexView()
        view.object_list = view.get_queryset()
        view.setup(request)

        self.assertNotIn("can_add_question",view.get_context_data())

    @override_settings(FAQ_SETTINGS=["no_category","logged_in_users_can_add_question"])
    def test_get_context_data_not_using_categories_logged_in_can_add(self):
        "gets context data correctly when not using categories and logged_in_users_can_add_question"
        request = RequestFactory()
        request = request.get(reverse("faq:index_view"))
        request.user=User.objects.create_user(username="jim",password="the")
        view = IndexView()
        view.object_list = view.get_queryset()
        view.setup(request)

        self.assertIn("can_add_question",view.get_context_data())

class CategoryDetailTestCase(TestCase):
    def setUp(self):
        models.Category.objects.create(name="cat1", description="descript")
        models.Category.objects.create(name="cat2", description="descript2")
        models.Category.objects.create(name="cat3")
    
    @override_settings(FAQ_SETTINGS=[])
    def test_get_context_data(self):
        request = RequestFactory().get(reverse("faq:category_detail",args=("cat1",)))
        view = CategoryDetail()
        view.object = view.get_object()
        view.setup(request)
        
        view.get_context_data()


from django.core.exceptions import PermissionDenied
from django.shortcuts import reverse
from django.views import generic
from django.contrib.auth.mixins import UserPassesTestMixin
from django.conf import settings
from . import models
from . import forms
# Create your views here.
class IndexView(generic.ListView):
    """
    this view depending on settings either displays all the categories as a list if the categories is enabled using the categories_list.html template

    if categories are not enabled it will then show a list of all the questions using questions_list.html template
    """
    def get_template_names(self):
        """if "no_category" render questions_list.html
            else render categories_list.html"""
        if "no_category" in settings.FAQ_SETTINGS:
            return "faq/questions_list.html"
        return "faq/categories_list.html"

    def get_queryset(self):
        if "no_category" in settings.FAQ_SETTINGS:
            return models.Question.objects.all()
        return models.Category.objects.all()

    def get_context_object_name(self,object_list):
        if "no_category" in settings.FAQ_SETTINGS:
            return "questions"
        return "categories"

    def get_context_data(self, **kwargs):
        context=super().get_context_data(**kwargs)
        # if not using categories
        if "no_category" in settings.FAQ_SETTINGS:
            context["can_add_question"]=False
            if "logged_in_users_can_add_question" in settings.FAQ_SETTINGS:
                if self.request.user.is_authenticated:
                    context["can_add_question"] = True
        return context

class CategoryDetail(generic.DetailView):
    """
    this view only runs when categories are enabled
    this view shows all the questions related to this category
    """
    model= models.Category
    template_name = "faq/category_detail.html"

    def get_context_data(self, *, object_list=None, **kwargs):
        context=super().get_context_data()
        if "logged_in_users_can_add_question" in settings.FAQ_SETTINGS:
            if self.request.user.is_authenticated:
                context["can_add_question"] = True
        return context

class AddQuestion(UserPassesTestMixin,generic.CreateView):
    """
    this view is for a user to add a question to the faq questions
    if the using categories then this will also add the current category to it
    the default setting is only to allow the superuser to add new questions
    to change this:
        A) override the testfunc method
        B) set staff_can_add_question in the FAQ_SETTINGS to True (allows any staff user to add a question)
        C) set authenticated_user_can_add_question in the FAQ_SETTINGS to True (allows any authenticated user to add a question)
    """
    model = models.Question
    fields = ['question']
    def test_func(self):
        # when authenticated_user_can_add_question in the FAQ_SETTINGS is set to True
        if "logged_in_users_can_add_question" in settings.FAQ_SETTINGS:
            if self.request.user.is_authenticated:
                return True
        return False

    def get_success_url(self):
       return self.question_url

    def form_valid(self, form):
        # if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            form=form.save(commit=False)
            form.category= models.Category.objects.get(slug=self.kwargs["slug"])
            form.save()
            self.question_url=form.get_absolute_url()
            return super().form_valid(form)
        else:
            form=form.save()
            self.question_url=form.get_absolute_url()
            return super().form_valid(form)

class QuestionDetail(generic.DetailView):
    model= models.Question
    template_name = "faq/question_detail.html"
    context_object_name = "question"
    def get_object(self, queryset=None):
        #if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            return self.model.objects.get(category__slug=self.kwargs["slug"],slug=self.kwargs["question"])
        else:
            return self.model.objects.get(slug=self.kwargs["slug"])

    def get_context_data(self, **kwargs):
        context=super().get_context_data()
        # if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            context['category_enabled']=True
        else:
            context['category_enabled']=False

        if "allow_multiple_answers" in settings.FAQ_SETTINGS:
            context['allow_multiple_answers'] = True
        else:
            context['allow_multiple_answers'] = False

        # check if logged in users are allowed to answer questions
        if "logged_in_users_can_answer_question" in settings.FAQ_SETTINGS:
            # check if user is logged in
            if self.request.user.is_authenticated:
                if "allow_multiple_answers" in settings.FAQ_SETTINGS:
                    context['can_answer_question'] = True
                else:
                    #if there is already one answer
                    if self.get_object().answer_set.count()>0:
                        context['can_answer_question'] = False
                    else:
                        context['can_answer_question'] = True
            else:
                context['can_answer_question'] = False
        else:
            context['can_answer_question'] = False



        # if using comments
        if "no_comments" not in settings.FAQ_SETTINGS:
            context["comments_allowed"]=True
            if "anonymous_user_can_comment" in settings.FAQ_SETTINGS:
                context['add_new_comment_allowed'] = True
            else:
                if self.request.user.is_authenticated:
                    context['add_new_comment_allowed'] = True
                else:
                    context['add_new_comment_allowed'] = False

            if "view_only_comments" in settings.FAQ_SETTINGS:
                context['add_new_comment_allowed'] = False
        else:
            context["comments_allowed"] = False

        # if can vote on answers
        if "no_votes" not in settings.FAQ_SETTINGS:
            # if can vote answer
            if "no_answer_votes" not in settings.FAQ_SETTINGS:
                context["can_vote_answer"] = True
            else:
                context["can_vote_answer"] = False

            if "no_question_votes" not in settings.FAQ_SETTINGS:
                context["can_vote_question"] = True
            else:
                context["can_vote_question"] = False
        else:
            context["can_vote_answer"] = False
            context["can_vote_question"] = False


        context["comment_form"] = forms.CommentForm()
        return context

class AddAnswer(UserPassesTestMixin,generic.CreateView):
    model = models.Answer
    fields = ["answer"]
    template_name = "faq/answer_form.html"

    def test_func(self):
        # when authenticated_user_can_add_question in the FAQ_SETTINGS is set to True
        if "logged_in_users_can_answer_question" in settings.FAQ_SETTINGS:
            if self.request.user.is_authenticated:
                if "allow_multiple_answers" in settings.FAQ_SETTINGS:
                    return True
                else:

                    # if using categories
                    if "no_category" not in settings.FAQ_SETTINGS:
                        question = models.Question.objects.get(category__slug=self.kwargs['category'],
                                                               slug=self.kwargs['question'])
                    else:
                        question = models.Question.objects.get(slug=self.kwargs['question'])

                    # if there is already one answer don't allow user to add answer
                    if question.answer_set.count() > 0:
                        return False
                    else:
                        return True
        return False
    
    def get_success_url(self):
        # if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            return reverse("faq:question_detail",args=(self.kwargs['category'],self.kwargs['question']) )
        else:
            return reverse("faq:question_detail",args=(self.kwargs['question'],) )

    def get_context_data(self, **kwargs):
        context=super().get_context_data()

        # if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            question = models.Question.objects.get(category__slug=self.kwargs['category'], slug=self.kwargs['question'])
        else:
            question = models.Question.objects.get(slug=self.kwargs['question'])

        context["question"]=question
        return context


    def form_valid(self, form):
        form=form.save(commit=False)

        # if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            question = models.Question.objects.get(category__slug=self.kwargs['category'], slug=self.kwargs['question'])
        else:
            question = models.Question.objects.get(slug=self.kwargs['question'])

        form.question=question
        form.save()
        return super().form_valid(form)

class AddComment(generic.CreateView):
    model = models.FAQComment
    form_class = forms.CommentForm
    template_name = "faq/comment_form.html"

    def get_question(self):
        # if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            question = models.Question.objects.get(category__slug=self.kwargs['category'], slug=self.kwargs['question'])
        else:
            question = models.Question.objects.get(slug=self.kwargs['question'])

        return question

    def get_success_url(self):
        return self.get_question().get_absolute_url()

    def get_context_data(self, **kwargs):
        context = super().get_context_data()
        context["question"]=self.get_question()
        return context

    def form_valid(self, form):
        form=form.save(commit=False)
        form.question=self.get_question()
        if self.request.user.is_authenticated:
            form.user=self.request.user
        form.save()
        return super().form_valid(form)

    def get(self,*args,**kwargs):
        if "view_only_comments" in settings.FAQ_SETTINGS:
            raise PermissionDenied("comments are view only at this time")
        if self.request.user.is_authenticated:
            pass
        else:
            if not "anonymous_user_can_comment" in settings.FAQ_SETTINGS:
                raise PermissionDenied("have to be logged in to comment")
        return super().get(*args,**kwargs)

    def post(self,*args,**kwargs):
        if "view_only_comments" in settings.FAQ_SETTINGS:
            raise PermissionDenied("comments are view only at this time")
        if self.request.user.is_authenticated:
            pass
        else:
            if not "anonymous_user_can_comment" in settings.FAQ_SETTINGS:
                raise PermissionDenied("have to be logged in to comment")
        return super().post(*args,**kwargs)
    
class VoteAnswerHelpful(UserPassesTestMixin,generic.FormView):
    form_class = forms.VoteForm
    template_name = "faq/vote_form.html"

    def get_success_url(self):
        return self.get_question().get_absolute_url()

    def get_question(self):
        # if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            question = models.Question.objects.get(category__slug=self.kwargs['category'], slug=self.kwargs['question'])
        else:
            question = models.Question.objects.get(slug=self.kwargs['question'])

        return question

    def get_answer(self):
        return models.Answer.objects.get(question=self.get_question(), slug=self.kwargs['answer'])

    def form_valid(self, form):
        # if already voted get vote otherwise create it
        if models.AnswerHelpful.objects.filter(answer=self.get_answer(), user=self.request.user).exists():
            helpful= models.AnswerHelpful.objects.get(answer=self.get_answer(), user=self.request.user)
        else:
            helpful = models.AnswerHelpful(answer=self.get_answer(), user=self.request.user)

        helpful.vote=form.cleaned_data['vote']
        helpful.save()
        self.get_answer().save()
        return super().form_valid(form)

    def test_func(self):
        if "no_answer_votes" in settings.FAQ_SETTINGS:
            return False
        elif "no_votes" in settings.FAQ_SETTINGS:
            return False
        return True

class VoteQuestionHelpful(UserPassesTestMixin,generic.FormView):
    form_class = forms.VoteForm
    template_name = "faq/vote_form.html"

    def get_success_url(self):
        return self.get_question().get_absolute_url()

    def get_question(self):
        # if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            question = models.Question.objects.get(category__slug=self.kwargs['category'], slug=self.kwargs['question'])
        else:
            question = models.Question.objects.get(slug=self.kwargs['question'])

        return question

    def form_valid(self, form):
        # if already voted get vote otherwise create it
        if models.QuestionHelpful.objects.filter(question=self.get_question(), user=self.request.user).exists():
            helpful= models.QuestionHelpful.objects.get(question=self.get_question(), user=self.request.user)
        else:
            helpful = models.QuestionHelpful(question=self.get_question(), user=self.request.user)

        helpful.vote=form.cleaned_data['vote']
        helpful.save()
        self.get_question().save()
        return super().form_valid(form)

    def test_func(self):
        if "no_question_votes" in settings.FAQ_SETTINGS:
            return False
        elif "no_votes" in settings.FAQ_SETTINGS:
            return False
        return True


import secrets

ALL_URL_CHARS = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
def create_random_slug(size=10):
    """amount of characters you want generated for random_slug"""
    res=''.join(secrets.choice(ALL_URL_CHARS) for _ in range(size))
    return str(res)



from django.contrib import admin
from .models import *
from django.conf import settings
from . import forms
# Register your models here.

class AnswerHelpfulAdmin(admin.ModelAdmin):
    list_display = ("vote","answer" ,"user")
    list_filter = ('vote',)
    search_fields = ['answer', "user"]

class QuestionHelpfulAdmin(admin.ModelAdmin):
    list_display = ("vote","question" ,"user")
    list_filter = ('vote',)
    search_fields = ['question', "user"]

class AnswerAdmin(admin.ModelAdmin):
    list_display = ("answer","question" ,"helpful", "not_helpful")
    list_filter = ('helpful', "not_helpful")
    search_fields = ['answer', "question"]
    readonly_fields = ('helpful',"not_helpful")

class CategoryAdmin(admin.ModelAdmin):
    list_display = ("name","slug" ,"description")
    search_fields = ['name', "description"]

class CommentAdmin(admin.ModelAdmin):
    list_display = ("comment","question" ,"user", "post_time")
    list_filter = ('question', "post_time")
    search_fields = ['comment', "question"]

class QuestionAdmin(admin.ModelAdmin):
    list_display = ("question","category" ,"slug","helpful", "not_helpful")
    list_filter = ('helpful', "not_helpful","category")
    search_fields = ["question"]
    readonly_fields = ('helpful',"not_helpful")

admin.site.register(Question,QuestionAdmin)
admin.site.register(Answer,AnswerAdmin)

# if category enabled
if "no_category" not in settings.FAQ_SETTINGS:
    admin.site.register(Category,CategoryAdmin)


# if comments are enabled
if "no_comments" not in settings.FAQ_SETTINGS:
    admin.site.register(FAQComment,CommentAdmin)

# if votes are enabled
if "no_votes" not in settings.FAQ_SETTINGS:
    # if answer votes are enabled
    if "no_answer_votes" not in settings.FAQ_SETTINGS:
        admin.site.register(AnswerHelpful,AnswerHelpfulAdmin)

    # if question votes are enabled
    if "no_question_votes" not in settings.FAQ_SETTINGS:
        admin.site.register(QuestionHelpful,QuestionHelpfulAdmin)


from django.db import models
from django.template.defaultfilters import slugify
from django.shortcuts import reverse
from django.conf import settings
from . import snippets
# Create your models here.
class Question(models.Model):
    category = models.ForeignKey("category",on_delete=models.SET_NULL,null=True,blank=True)
    question = models.CharField(max_length=150, unique=True)
    slug     = models.SlugField(max_length=150, unique=True)
    helpful = models.IntegerField(default=0)
    not_helpful = models.IntegerField(default=0)

    def get_helpful(self):
        return QuestionHelpful.objects.filter(question=self,vote=True).count()

    def get_not_helpful(self):
        return QuestionHelpful.objects.filter(question=self,vote=False).count()

    def __str__(self):
        return self.question

    def save(self,*args, **kwargs):
        self.slug=slugify(self.question)
        self.helpful=self.get_helpful()
        self.not_helpful=self.get_not_helpful()
        return super().save(*args, **kwargs)

    def get_absolute_url(self):
        # if using categories
        if "no_category" not in settings.FAQ_SETTINGS:
            return reverse("faq:question_detail", args=(self.category.slug, self.slug))
        else:
            return reverse("faq:question_detail", args=(self.slug,))

class Answer(models.Model):
    question = models.ForeignKey(Question,on_delete=models.CASCADE)
    answer   = models.TextField()
    slug = models.SlugField(max_length=10)
    helpful = models.IntegerField(default=0)
    not_helpful = models.IntegerField(default=0)

    def get_helpful(self):
        return AnswerHelpful.objects.filter(answer=self,vote=True).count()

    def get_not_helpful(self):
        return AnswerHelpful.objects.filter(answer=self,vote=False).count()

    def __str__(self):
        return self.answer

    class Meta:
        order_with_respect_to = 'question'

    def save(self,*args,**kwargs):
        # if first time saving add a new slug
        if not self.pk:
            new_slug= snippets.create_random_slug(5)
            while Answer.objects.filter(slug=new_slug,answer=self.answer).exists():
                new_slug = snippets.create_random_slug()
            self.slug=new_slug
        self.helpful=self.get_helpful()
        self.not_helpful=self.get_not_helpful()
        super().save(*args,**kwargs)

class Category(models.Model):
    name        = models.CharField(max_length=50,unique=True)
    description = models.TextField()
    slug        =models.SlugField(max_length=50,unique=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "categories"

    def save(self,*args, **kwargs):
        self.slug=slugify(self.name)
        return super().save(*args, **kwargs)

class FAQComment(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,on_delete=models.CASCADE,null=True)
    comment = models.TextField()
    question = models.ForeignKey(Question,on_delete=models.CASCADE)
    post_time = models.DateTimeField(auto_now_add=True)
    def __str__(self):
        return self.comment

    class Meta:
        ordering = ['question', '-post_time']

class AnswerHelpful(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,on_delete=models.CASCADE)
    answer = models.ForeignKey(Answer,on_delete=models.CASCADE)
    vote = models.BooleanField()

    def __str__(self):
        if self.vote:
            vote_var = 'like'
        else:
            vote_var = 'dislike'

        return str(self.answer) + '- ' + vote_var

    class Meta:
        ordering = ['answer','vote']

class QuestionHelpful(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,on_delete=models.CASCADE)
    question = models.ForeignKey(Question,on_delete=models.CASCADE)
    vote = models.BooleanField()

    def __str__(self):
        if self.vote:
            vote_var = 'like'
        else:
            vote_var = 'dislike'

        return str(self.question) + '- ' + vote_var

    class Meta:
        ordering = ['question','vote']


from django.urls import path
from . import views
from django.conf import settings

app_name = 'faq'

urlpatterns = [
    path("", views.IndexView.as_view(), name="index_view"),
]

#if using categories
if "no_category" not in settings.FAQ_SETTINGS:
    urlpatterns += [
        path("<slug:slug>/", views.CategoryDetail.as_view(), name="category_detail"),
        path("<slug:slug>/add/question/", views.AddQuestion.as_view(), name="add_question"),
        path("<slug:slug>/<slug:question>/", views.QuestionDetail.as_view(), name="question_detail"),
        path("<slug:category>/<slug:question>/answer/", views.AddAnswer.as_view(), name="answer_question"),
    ]
else:
    urlpatterns += [
        path("<slug:slug>/", views.QuestionDetail.as_view(), name="question_detail"),
        path("<slug:question>/answer/", views.AddAnswer.as_view(), name="answer_question"),
        path("add/question", views.AddQuestion.as_view(), name="add_question"),
    ]

#if using comments
if "no_comments" not in settings.FAQ_SETTINGS:
    # if using categories
    if "no_category" not in settings.FAQ_SETTINGS:
        urlpatterns += [
            path("<slug:category>/<slug:question>/add/comment/", views.AddComment.as_view(), name="add_comment"),
        ]
    else:
        urlpatterns += [
            path("<slug:question>/add/comment/", views.AddComment.as_view(), name="add_comment"),
        ]


#if using votes
if "no_votes" not in settings.FAQ_SETTINGS:
    # if using categories
    if "no_category" not in settings.FAQ_SETTINGS:
        urlpatterns += [
            path("<slug:category>/<slug:question>/<slug:answer>/vote/", views.VoteAnswerHelpful.as_view(),
                 name="vote_answer"),
            path("<slug:category>/<slug:question>/vote/", views.VoteQuestionHelpful.as_view(), name="vote_question")
        ]
    else:
        urlpatterns += [
            path("<slug:question>/<slug:answer>/vote/", views.VoteAnswerHelpful.as_view(),
                 name="vote_answer"),
            path("<slug:question>/vote/", views.VoteQuestionHelpful.as_view(), name="vote_question")
        ]



from django import forms
from django.conf import settings
from . import models

class CategoryForm(forms.ModelForm):
    """
    creates a form to create and update a category
    if you don't plan on using a description for your category
    then add this to the FAQ_SETTINGS dictionary inside of the settings file
    "category_description":False,
    """
    class Meta:
        model = models.Category
        fields=["name","slug"]

    def __init__(self, *args, **kwargs):
        super(CategoryForm, self).__init__(*args, **kwargs)

        # if using category descriptions add to form
        if not "no_category_description" in settings.FAQ_SETTINGS:
            self.fields["description"] = forms.CharField()

class CommentForm(forms.ModelForm):
    class Meta:
        model = models.FAQComment
        fields = ["comment"]

class VoteForm(forms.Form):
    vote = forms.BooleanField(label="helpful? ",required=False)
